%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Download HMM toolbox from:
% https://github.com/OHBA-analysis/HMM-MAR

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% HMM Simulation 1: variance changes

K=3; % num states

% state 1 variance increased in 1: 
Sigma = [2 0 0 0 
         0 1 0 0
         0 0 1 0
         0 0 0 1];
k1 = randn(1000,4)*chol(Sigma);

% state 2 variance increased in 2: 
Sigma = [1 0 0 0
         0 2 0 0
         0 0 1 0
         0 0 0 1];
k2 = randn(1000,4)*chol(Sigma);

% state 3 variance increased in 3: 
Sigma = [1 0 0 0
         0 1 0 0
         0 0 2 0
         0 0 0 1];
k3 = randn(1000,4)*chol(Sigma);

% concatenate temporally:
simdata = [k1;k2;k3];
T = [length(k1);length(k2);length(k3)];
truepath=[ones(size(k1));2*ones(size(k2));3*ones(size(k3))];

%% infer HMM with 3 states:

options = struct();
options.K = K; 
options.Fs = 1; 
options.covtype = 'full';
options.order = 0;
options.zeromean = 1;
options.verbose = 1;
[hmm_env,Gamma_env] = hmmmar(simdata,T,options);
    
%% compute state covariances

for k=1:hmm_env.K
    hmm_env.state(k).Cov=hmm_env.state(k).Omega.Gam_rate / hmm_env.state(k).Omega.Gam_shape;
end

%% do plots

% Plot simulated data
figure, subplot(5,3,1:3), plot(simdata), axis tight; set(gca,'Title',text('String','Simulated data'))

% Plot simulated covariance matrices
subplot(5,3,4), imagesc(cov(k1)), colormap('gray')
subplot(5,3,5), imagesc(cov(k2)), colormap('gray'), set(gca,'Title',text('String','Simulated covariance'))
subplot(5,3,6), imagesc(cov(k3)), colormap('gray')

% Plot inferred covariance matrices
subplot(5,3,7), imagesc(hmm_env.state(1).Cov), colormap('gray')
subplot(5,3,8), imagesc(hmm_env.state(2).Cov), colormap('gray'), set(gca,'Title',text('String','Inferred covariance'))
subplot(5,3,9), imagesc(hmm_env.state(3).Cov), colormap('gray')

%% Plot state path

% Plot true state path
subplot(5,3,10:12); hold on
plot(truepath), set(gca,'Title',text('String','True state path'))
set(gca,'ylim',[0.5 3.5]); ylabel('state #')

%[Gamma,Xi] = hmmdecode(simdata,T,hmm_env,0) 
[viterbipath] = hmmdecode(simdata,T,hmm_env,1);

% Plot inferred state path
subplot(5,3,13:15); hold on
plot(viterbipath), set(gca,'Title',text('String','Inferred state path'))
set(gca,'ylim',[0.5 3.5]); ylabel('state #')

set(gcf,'pos',[100 100 600 800],'color','w')



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% HMM Simulation 2: covariance changes


% state 1 covariance increased between 1 & 2: 
Sigma = [1    0.5  0   0
         0.5  1    0   0
         0    0    1   0
         0    0    0   1];
k1 = randn(1000,4)*chol(Sigma);

% state 2 covariance increased between 1 & 3: 
Sigma = [1    0   0.5  0
         0    1   0    0
         0.5  0   1    0
         0    0   0    1];
k2 = randn(1000,4)*chol(Sigma);

% state 3 covariance increased between 2 & 4: 
Sigma = [1   0     0    0
         0   1     0    0.5
         0   0     1    0
         0   0.5   0    1];
k3 = randn(1000,4)*chol(Sigma);

% concatenate temporally:
simdata = [k1;k2;k3];
T = [length(k1);length(k2);length(k3)];
truepath=[ones(size(k1));2*ones(size(k2));3*ones(size(k3))];

%% infer HMM with 3 states:

options = struct();
options.K = K; 
options.Fs = 1; 
options.covtype = 'full';
options.order = 0;
options.zeromean = 1;
options.verbose = 1;
[hmm_env,Gamma_env] = hmmmar(simdata,T,options);
    
%% compute state covariances

for k=1:hmm_env.K
    hmm_env.state(k).Cov=hmm_env.state(k).Omega.Gam_rate / hmm_env.state(k).Omega.Gam_shape;
end

%% do plots

% Plot simulated data
figure, subplot(5,3,1:3), plot(simdata), axis tight; set(gca,'Title',text('String','Simulated data'))

% Plot simulated covariance matrices
subplot(5,3,4), imagesc(cov(k1)), colormap('gray')
subplot(5,3,5), imagesc(cov(k2)), colormap('gray'), set(gca,'Title',text('String','Simulated covariance'))
subplot(5,3,6), imagesc(cov(k3)), colormap('gray')

% Plot inferred covariance matrices
subplot(5,3,7), imagesc(hmm_env.state(1).Cov), colormap('gray')
subplot(5,3,8), imagesc(hmm_env.state(2).Cov), colormap('gray'), set(gca,'Title',text('String','Inferred covariance'))
subplot(5,3,9), imagesc(hmm_env.state(3).Cov), colormap('gray')

%% Plot state path

% Plot true state path
subplot(5,3,10:12); hold on
plot(truepath), set(gca,'Title',text('String','True state path'))
set(gca,'ylim',[0.5 3.5]); ylabel('state #')

%[Gamma,Xi] = hmmdecode(simdata,T,hmm_env,0) 
[viterbipath] = hmmdecode(simdata,T,hmm_env,1);

% Plot inferred state path
subplot(5,3,13:15); hold on
plot(viterbipath), set(gca,'Title',text('String','Inferred state path'))
set(gca,'ylim',[0.5 3.5]); ylabel('state #')

set(gcf,'pos',[100 100 600 800],'color','w')

